import { Component } from "@angular/core";
import { Router } from "@angular/router";
import { TableApiService } from "src/app/_services/table-api.service";
import { BuildingService } from "../service/monitoring-api.service";
import Swal from "sweetalert2";
import { LoginService } from "../service/login.service";
import { ToastrService } from "ngx-toastr";

@Component({
  selector: "app-list-monitoring",
  templateUrl: "./list-monitoring.component.html",
  styleUrls: ["./list-monitoring.component.css"],
})
export class ListMonitoringComponent {
  public breadcrumb: any;
  data: any;
  filteredRows: any[];
  searchTerm: string = "";
  rows: any = [];

  constructor(
    private router: Router,
    private monitoringApiService: BuildingService,
    private authService: LoginService,
    private toastr: ToastrService,
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.breadcrumb = {
      mainlabel: "List Monitoring",
      links: [
        {
          name: "Home",
          isLink: false,
        },
        {
          name: "List Monitoring",
          isLink: false,
        },
      ],
    };
    this.fetchData();
  }

  fetchData() {
    this.monitoringApiService.getBuildingRoomList().subscribe((res) => {
      this.data = res;
      this.filteredRows = this.data.results.data;
    });
  }

  filterRows() {
    if (!this.searchTerm) {
      this.filteredRows = [...this.data.results.data];
    } else {
      const searchTermLC = this.searchTerm.toLowerCase();
      this.filteredRows = this.data.results.data.filter((row) =>
        this.rowContainsSearchTerm(row, searchTermLC)
      );
    }
  }

  rowContainsSearchTerm(row: any, searchTermLC: string): boolean {
    return (
      row.buildingEntity.name.toLowerCase().includes(searchTermLC) ||
      row.roomEntity.name.toLowerCase().includes(searchTermLC) ||
      row.statusEntity.name.toLowerCase().includes(searchTermLC)
    );
  }

  viewRow(row) {
    this.router.navigate(["/list-monitoring/view", row.id]);
  }

  editRow(row) {
    this.router.navigate(["/list-monitoring/edit", row.id]);
  }

  deleteRow(row) {
    const confirmDelete = confirm("Are you sure you want to delete this item?");
    if (confirmDelete) {
      this.monitoringApiService.deleteRoomBuilding(row.id).subscribe(
        (res) => {
          this.fetchData();
          this.toastr.success("Success", "Delete Completed.", {
            timeOut: 2000,
            closeButton: true,
          });
        },
        (error) => {
          console.error(error);
          this.toastr.error("Error", "Data sedang digunakan!", {
            timeOut: 2000,
            closeButton: true,
          });
        }
      );
    }
  }

  onTouchStart(event: Event) {
    event.preventDefault(); // Add this if necessary
  }
}
