import { Component, ElementRef, ViewChild } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { ChartOptions, ChartType, ChartDataset } from "chart.js";
import { EnergyMonitoringService } from "../../service/energy-monitoring.service";
import { CurrencyPipe } from "@angular/common";
import { LoginService } from "../../service/login.service";
import { DeviceService } from "../../service/device.service";
import { color } from "echarts";

@Component({
  selector: "app-detail",
  templateUrl: "./detail.component.html",
  styleUrls: ["./detail.component.css"],
  providers: [CurrencyPipe],
})
export class DetailComponent {
  data: any;
  mode: string;
  breadcrumb: any;
  donutChart1: any;
  donutChart2: any;

  // temperature
  days = [
    { name: "Sun", isToday: false },
    { name: "Mon", isToday: false },
    { name: "Tue", isToday: false },
    { name: "Wed", isToday: false },
    { name: "Thu", isToday: false },
    { name: "Fri", isToday: false },
    { name: "Sat", isToday: false },
  ];
  roomTemperature: number = 0;
  roomHumidity: number = 0;
  valueAirQuality: number = 0;
  currentDate: string;
  currentTime: string;
  chartOption: any;
  chartOptionBar: any;

  // chart bar
  public barChartOptions: ChartOptions = {
    responsive: true,
    animation: {
      duration: 0,
    },
    scales: {
      x: {
        stacked: true,
      },
      y: {
        stacked: true,
        beginAtZero: true,
      },
    },
  };

  public barChartLabels: string[] = [];
  public barChartType: ChartType = "bar";
  public barChartLegend = true;
  public barChartData: number[] = [];
  public barChartData2: number[] = [];
  //..........................

  // integrasi
  electric: any;
  water: any;
  device: any;
  room: any;
  topCard: any;
  summaryCost: any;
  airQuality: any;
  temperature: any;
  deviceCategory: any;
  chartKwhWater: any;
  paramsId: any;
  //......

  constructor(
    private route: ActivatedRoute,
    private energyMonitoringService: EnergyMonitoringService,
    private currencyPipe: CurrencyPipe,
    private router: Router,
    private authService: LoginService,
    private deviceService: DeviceService
  ) {}

  get formattedSummaryCost(): string {
    return this.currencyPipe.transform(
      this.summaryCost.summary_cost,
      "IDR",
      "symbol",
      "1.0-0"
    );
  }

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.route.data.subscribe((data) => {
      this.mode = data.mode;
    });
    this.breadcrumbLink();
    this.route.params.subscribe((params) => {
      const buildingId = params["id"];
      this.paramsId = params["id"];
      if (buildingId) {
        this.dataEnergyMonitoringTopCard(buildingId);
        this.dataEnergyMonitoringSummary(buildingId);
        this.dataEnergyMonitoringSAir(buildingId);
        this.dataEnergyDeviceCategory(buildingId);
        this.dataEnergyChartKwhWater(buildingId);
        this.dataDeviceIr();
        this.getAirQualityData();
      }
    });
  }

  getAirQualityData() {
    this.valueAirQuality = Math.floor(Math.random() * 501);
    this.chartOption = {
      series: [
        {
          type: "gauge",
          startAngle: 180,
          endAngle: 0,
          center: ["50%", "75%"],
          radius: "130%", // Adjust the radius to make the chart smaller
          min: 0,
          max: 500,
          splitNumber: 5,
          axisLine: {
            lineStyle: {
              width: 20,
              color: [
                [1 / 6, "#6BD67C"], // 0 - 50: Green
                [2 / 6, "#A2D856"], // 51 - 100: Light Green
                [3 / 6, "#FFE45E"], // 101 - 150: Yellow
                [4 / 6, "#FFA644"], // 151 - 200: Orange
                [5 / 6, "#FF6E76"], // 201 - 300: Light Red
                [6 / 6, "#E768A7"], // 301 - 500: Purple
              ],
            },
          },
          pointer: {
            icon: "path://M12.8,0.7l12,40.1H0.7L12.8,0.7z",
            length: "12%",
            width: 15,
            offsetCenter: [0, "-50%"],
            itemStyle: {
              color: "auto",
            },
          },
          axisTick: {
            length: 10,
            lineStyle: {
              color: "auto",
              width: 2,
            },
          },
          splitLine: {
            length: 15,
            lineStyle: {
              color: "auto",
              width: 5,
            },
          },
          axisLabel: {
            color: "#464646",
            fontSize: 16, // Adjust font size
            distance: -40, // Adjust distance
            rotate: "tangential",
            formatter: function (value: number) {
              // return Math.round(value) === 100 ? "100" : Math.round(value) + "";
              return value.toString();
            },
          },
          title: {
            offsetCenter: [0, " 15%"],
            fontSize: 18, // Adjust font size
          },
          detail: {
            fontSize: 24, // Adjust font size
            offsetCenter: [0, "-25%"], // Adjust offset
            valueAnimation: true,
            formatter: function (value: number) {
              return Math.round(value) + "";
            },
            color: "#242222",
          },
          data: [
            {
              value: this.valueAirQuality,
              name: this.getGradeName(this.valueAirQuality),
            },
          ],
        },
      ],
    };
  }

  getGradeName(value: number): string {
    if (value <= 50) {
      return "Good";
    } else if (value <= 100) {
      return "Moderate";
    } else if (value <= 150) {
      return "Unhealthy for Sensitive Groups";
    } else if (value <= 200) {
      return "Unhealthy";
    } else if (value <= 300) {
      return "Very Unhealthy";
    } else {
      return "Hazardous";
    }
  }

  breadcrumbLink() {
    this.breadcrumb = {
      mainlabel: "Energy Monitoring",
      linkBack: "/monitoring",
      isLinkBack: true,
      links: [
        {
          name: "Dashboard",
          isLink: false,
          link: "",
        },
        {
          name: "Energy Monitoring",
          isLink: false,
          link: "",
        },
      ],
    };
  }
  //integrasi
  dataEnergyMonitoringTopCard(buildingId) {
    this.energyMonitoringService
      .getDashboardTopCard(buildingId)
      .subscribe((res) => {
        this.topCard = res.data;
      });
  }

  dataDeviceIr() {
    const today = new Date().getDay();
    this.days[today].isToday = true;
    const now = new Date();
    this.currentDate = now.toLocaleDateString();
    this.currentTime = now.toLocaleTimeString();
    this.deviceService.getDeviceStatus(452).subscribe((res) => {
      console.log(res);
      this.roomTemperature = res.result[0].value;
      this.roomHumidity = res.result[1].value;
    });
  }

  dataEnergyMonitoringSummary(buildingId) {
    this.energyMonitoringService
      .getDashboardSummary(buildingId)
      .subscribe((res) => {
        this.summaryCost = res.data;
      });
  }

  dataEnergyMonitoringSAir(buildingId) {
    this.energyMonitoringService
      .getDashboardAirQuality(buildingId)
      .subscribe((res) => {
        this.airQuality = res.data;
      });
  }

  dataEnergyDeviceCategory(buildingId) {
    this.energyMonitoringService
      .getDashboardDeviceCategory(buildingId)
      .subscribe((res) => {
        this.deviceCategory = res.data[0];
      });
  }

  dataEnergyChartKwhWater(buildingId) {
    this.energyMonitoringService
      .getDashboardChartKwhWater(buildingId)
      .subscribe((res) => {
        this.chartKwhWater = res.data;
        console.log(this.chartKwhWater);
        this.chartKwhWater.forEach((entry) => {
          this.barChartData.push(entry.kwh.toFixed(1));
          this.barChartData2.push(entry.cost);
          this.barChartLabels.push(entry.day);
        });

        this.chartOptionBar = {
          tooltip: {
            trigger: "axis",
            axisPointer: {
              type: "shadow",
            },
          },
          legend: {},
          grid: {
            left: "3%",
            right: "4%",
            bottom: "3%",
            containLabel: true,
          },
          xAxis: [
            {
              type: "category",
              data: this.barChartLabels,
            },
          ],
          yAxis: [
            {
              type: "value",
            },
          ],
          series: [
            {
              name: "KWH Consumption",
              type: "bar",
              color: "#ffffff",
              stack: "Ad",
              emphasis: {
                focus: "series",
              },
              data: this.barChartData,
            },
            {
              name: "Cost Consumption",
              type: "bar",
              stack: "Ad",
              color: "#37a647",
              emphasis: {
                focus: "series",
              },
              data: this.barChartData2,
            },
          ],
        };
      });
  }

  aggregateKwhWeekly(data: any[]): number[] {
    const weeks = [];
    let weekSum = 0;
    let dayCount = 0;

    data.forEach((entry, index) => {
      weekSum += entry.kwh;
      dayCount++;
      if (dayCount === 7 || index === data.length - 1) {
        weeks.push(weekSum);
        weekSum = 0;
        dayCount = 0;
      }
    });

    return weeks;
  }

  seeMore(paramsId) {
    this.router.navigate(["/monitoring/control-device-see-more/", paramsId]);
  }
}
